#include "GPIO.h"
#include "OWI.h"
#include "Slave/OWI.h"
#include "Driver/Arduino.h"
// #include <Wire.h>
#include "Adafruit_SHT31.h"
#include "Adafruit_CCS811.h"
#include "benchmark.h"

/** One-Wire Slave Device Bus Manager. */
Slave::OWI<BOARD::D2> owi(Arduino::FAMILY_CODE);

Adafruit_SHT31 sht31 = Adafruit_SHT31();
Adafruit_CCS811 ccs;

struct scratchpad_t {
  float temperature;    //!< Temperature reading
  float humidity;    //!<
  float co2;   //!<
  float tvoc;  //!<
  uint8_t reserved[3];    //!< Reserved.
} __attribute__((packed));


scratchpad_t scratchpad = {
  0.0,      //!< 85 C default temperature,
  0.0,       //!< 75 C high trigger, and
  0.0,       //!< 70 C low trigger
  0.0,       //!< 10 bits conversion
  { 0, 0, 0 }     //!< Reserved
};

/**
 * Setup One-Wire Remote Arduino Slave Device. Set alarm
 * state to allow alarm search before reading analog input.
 */
void setup()
{
  owi.alarm(true);
  // uint8_t ROM[] = {Arduino::FAMILY_CODE, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x11}; = Adresse 3 Achtung Binäre obwohl x
  uint8_t ROM[] = {Arduino::FAMILY_CODE, 0xFF, 0x00, 0x00, 0x00, 0x01, 0x11};        // *************** hier statische Adresse für Slave angeben ****************** 
  owi.rom(ROM);

  sht31.begin(0x44);
  ccs.setDriveMode(CCS811_DRIVE_MODE_60SEC);
  ccs.begin();
  //calibrate temperature sensor
  //while(!ccs.available());
  float temp = ccs.calculateTemperature();
  ccs.setTempOffset(temp - 25.0);
}

/**
 * Service incoming one-wire rom and remote arduino functions.
 */
void loop()
{
  // Additional sketch code could be placed here before
  // polling one-wire bus for commands
  if (!owi.rom_command()) return;
  
  const size_t BUF_MAX = 32;
  uint8_t buf[BUF_MAX];
  uint16_t value;
  uint8_t pin;
  uint8_t mode;
  uint8_t* bp;
  uint8_t count;
  uint8_t res;
  uint8_t duty;
  float t;
  float h;
  float co2;
  float tvoc;

  // Read and dispatch remote arduino commands
  switch (owi.read_command()) {
  case Arduino::PIN_MODE:
    pin = owi.read(6);
    mode = owi.read(2);
    pinMode(pin, mode);
    break;
  case Arduino::DIGITAL_READ:
    pin = owi.read(6);
    value = digitalRead(pin);
    owi.write(value, 1);
    break;
  case Arduino::DIGITAL_WRITE:
    pin = owi.read(6);
    value = owi.read(1);
    digitalWrite(pin, value);
    break;
  case Arduino::ANALOG_READ:
    pin = owi.read(6);
    value = analogRead(pin);
    owi.alarm(value > 512);
    owi.write(&value, sizeof(value));
    break;
  case Arduino::ANALOG_WRITE:
    pin = owi.read(6);
    duty = owi.read();
    analogWrite(pin, duty);
    break;
  case Arduino::SRAM_READ:
    owi.read(&bp, sizeof(bp));
    count = owi.read();
    owi.write(bp, count);
    break;
  case Arduino::SRAM_WRITE:
    owi.read(&bp, sizeof(bp));
    count = owi.read();
    owi.read(bp, count + 1);
    break;
  case Arduino::EEPROM_READ:
    owi.read(&bp, sizeof(bp));
    count = owi.read();
    owi.crc(0);
    while (count--) owi.write(eeprom_read_byte(bp++));
    owi.write(owi.crc());
    break;
  case Arduino::EEPROM_WRITE:
    owi.read(&bp, sizeof(bp));
    count = owi.read();
    res = 0b00;
    if (count > BUF_MAX) {
      while (count--) owi.read();
      owi.read();
    }
    else if (owi.read(buf, count)) {
      eeprom_write_block(buf, bp, count);
      res = 0b10;
    }
    owi.write(res, 2);
    break;
  case Arduino::DIGITAL_PINS:
    owi.write(NUM_DIGITAL_PINS, 6);
    break;
  case Arduino::ANALOG_PINS:
    owi.write(NUM_ANALOG_INPUTS, 6);
    break;
  case Arduino::READ_SENSORS:
    t = sht31.readTemperature();
    h = sht31.readHumidity();
    if(!ccs.readData()){
        co2 = ccs.geteCO2();
        tvoc = ccs.getTVOC();
    }
    else{
      co2 = -1000;
      tvoc = -1000;
    }
    scratchpad.temperature = t;
    scratchpad.humidity = h;
    scratchpad.co2 = co2;
    scratchpad.tvoc = tvoc;
    break;
 case Arduino::READ_SCRATCHPAD:
    owi.write(&scratchpad, sizeof(scratchpad));
    break;
  }
}
